@@ -3,7 +3,7 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = "0.10";
+our $VERSION = "0.11";
 
 use Scalar::Util ();
 
@@ -30,8 +30,8 @@ sub new {
     return bless \%args => $class;
 }
 
-sub name()       { my $self = shift; return $self->{name} }
-sub type()       { my $self = shift; return $self->{type} }
+sub name()       { my $self = shift; return $self->{name} // '' }
+sub type()       { my $self = shift; return $self->{type} // '' }
 sub default()    { my $self = shift; return $self->{default} } ## no critic (ProhibitBuiltinHomonyms)
 sub coerce()     { my $self = shift; return $self->{coerce} }
 sub optional()   { my $self = shift; return !!$self->{optional} }
@@ -40,6 +40,11 @@ sub named()      { my $self = shift; return !!$self->{named} }
 sub positional() { my $self = shift; return !$self->{named} }
 sub invocant()   { my $self = shift; return !!$self->{invocant} }
 
+sub has_name()    { my $self = shift; return defined $self->{name} }
+sub has_type()    { my $self = shift; return defined $self->{type} }
+sub has_default() { my $self = shift; return defined $self->{default} }
+sub has_coerce()  { my $self = shift; return defined $self->{coerce} }
+
 sub set_name      { my ($self, $v) = @_; $self->{name}     = $v; return $self }
 sub set_type      { my ($self, $v) = @_; $self->{type}     = $v; return $self }
 sub set_default   { my ($self, $v) = @_; $self->{default}  = $v; return $self }
@@ -62,11 +67,19 @@ sub is_same_interface {
 
     return unless Scalar::Util::blessed($other) && $other->isa('Sub::Meta::Param');
 
-    return unless defined $self->name ? defined $other->name && $self->name eq $other->name
-                                      : !defined $other->name;
+    if ($self->has_name) {
+        return unless $self->name eq $other->name
+    }
+    else {
+        return if $other->has_name
+    }
 
-    return unless defined $self->type ? defined $other->type && $self->type eq $other->type
-                                      : !defined $other->type;
+    if ($self->has_type) {
+        return unless $self->type eq $other->type
+    }
+    else {
+        return if $other->has_type
+    }
 
     return unless $self->optional eq $other->optional;
 
@@ -81,11 +94,11 @@ sub is_same_interface_inlined {
     my @src;
     push @src => sprintf("Scalar::Util::blessed(%s) && %s->isa('Sub::Meta::Param')", $v, $v);
 
-    push @src => defined $self->name ? sprintf("defined %s->name && '%s' eq %s->name", $v, "@{[$self->name]}", $v)
-                                     : sprintf('!defined %s->name', $v);
+    push @src => $self->has_name ? sprintf("'%s' eq %s->name", $self->name, $v)
+                                 : sprintf('!%s->has_name', $v);
 
-    push @src => defined $self->type ? sprintf("defined %s->type && '%s' eq %s->type", $v, "@{[$self->type]}", $v)
-                                     : sprintf('!defined %s->type', $v);
+    push @src => $self->has_type ? sprintf("'%s' eq %s->type", "@{[$self->type]}", $v)
+                                 : sprintf('!%s->has_type', $v);
 
     push @src => sprintf("'%s' eq %s->optional", $self->optional, $v);
 
@@ -155,18 +168,38 @@ Constructor of C<Sub::Meta::Param>.
 
 =head3 name
 
+  method name() => Str
+
 variable name, e.g. C<$msg>, C<@list>.
 
-=head3 set_name(Str $name)
+=head3 has_name
+
+  method has_name() => Bool
+
+Whether Sub::Meta::Param has name or not.
+
+=head3 set_name($name)
+
+  method set_name(Str $name) => $self
 
 Setter for C<name>.
 
 =head3 type
 
+  method type() => Any
+
 Any type constraints, e.g. C<Str>.
 
+=head3 has_type
+
+  method has_type() => Bool
+
+Whether Sub::Meta::Param has type or not.
+
 =head3 set_type($type)
 
+  method set_type(Any $type) => $self
+
 Setter for C<type>.
 
 =head3 isa_
@@ -179,77 +212,123 @@ The alias of C<set_type>
 
 =head3 default
 
+  method default() => Any
+
 default value, e.g. C<"HELLO">, C<sub { ... }>
 
+=head3 has_default
+
+  method has_default() => Bool
+
+Whether Sub::Meta::Param has default or not.
+
 =head3 set_default($default)
 
+  method set_default(Any $default) => $self
+
 Setter for C<default>.
 
 =head3 coerce
 
+  method coerce() => Any
+
 A boolean value indicating whether to coerce. Default to false.
 
-=head3 set_coerce($bool)
+=head3 has_coerce
+
+  method has_coerce() => Bool
+
+Whether Sub::Meta::Param has coerce or not.
+
+=head3 set_coerce($coerce)
+
+  method set_coerce(Any $coerce) => $self
 
 Setter for C<coerce>.
 
 =head3 optional
 
+  method optional() => Bool
+
 A boolean value indicating whether to optional. Default to false.
 This boolean is the opposite of C<required>.
 
 =head3 set_optional($bool=true)
 
+  method set_optional(Bool $bool=true) => $self
+
 Setter for C<optional>.
 
 =head3 required
 
+  method required() => Bool
+
 A boolean value indicating whether to required. Default to true.
 This boolean is the opposite of C<optional>.
 
 =head3 set_required($bool=true)
 
+  method set_required(Bool $bool=true) => $self
+
 Setter for C<required>.
 
 =head3 named
 
+  method named() => Bool
+
 A boolean value indicating whether to named arguments. Default to false.
 This boolean is the opposite of C<positional>.
 
 =head3 set_named($bool=true)
 
+  method set_named(Bool $bool=true) => $self
+
 Setter for C<named>.
 
 =head3 positional
 
+  method positional() => Bool
+
 A boolean value indicating whether to positional arguments. Default to true.
 This boolean is the opposite of C<positional>.
 
 =head3 set_positional($bool=true)
 
+  method set_positional(Bool $bool=true) => $self
+
 Setter for C<positional>.
 
 =head3 invocant
 
+  method invocant() => Bool
+
 A boolean value indicating whether to be invocant. Default to false.
 
 =head3 set_invocant($bool=true)
 
+  method set_invocant(Bool $bool=true) => $self
+
 Setter for C<invocant>.
 
 =head2 METHODS
 
 =head3 is_same_interface($other_meta)
 
+  method is_same_interface(InstanceOf[Sub::Meta::Param] $other_meta) => Bool
+
 A boolean value indicating whether C<Sub::Meta::Param> object is same or not.
 Specifically, check whether C<name>, C<type>, C<optional> and C<named> are equal.
 
 =head3 is_same_interface_inlined($other_meta_inlined)
 
+  method is_same_interface_inlined(InstanceOf[Sub::Meta::Param] $other_meta) => Str
+
 Returns inlined C<is_same_interface> string.
 
 =head3 display
 
+  method display() => Str
+
 Returns the display of Sub::Meta::Param:
 
     use Sub::Meta::Param;
